if exists("b:did_ftplugin")
  finish
endif

setlocal lisp
setlocal tabstop=2
setlocal softtabstop=2
setlocal shiftwidth=2
setlocal comments=:;,:;;,:;;;
setlocal commentstring=;%s
setlocal nocindent
setlocal expandtab

" Sets the local leader to 's' for use in
" mappings within the plugin/buffer
if !exists("g:slope_leader")
  let g:slope_leader="s"
endif
let maplocalleader=g:slope_leader


if !exists("g:slope_command")
  let g:slope_command = "slope"
endif

function! SaveAndRunSelection(startline,endline)
  let lines = getline(a:startline, a:endline)
  if len(lines) == 0
    return ''
  endif
  let lines[-1] = lines[-1][: column_end - 2]
  let lines[0] = lines[0][column_start - 1:]
  let tmpdir = tempname()
  let tmpfile = tmpdir . "/prog.slo"
  call mkdir(tmpdir)
  call writefile(lines, tmpfile,"fa") 
	" the interpreter can be overridden in .vim by setting g:slope_command
	" to something else
  execute "!" . g:slope_command . " " . tmpfile
	return
endfunction

" Function to run the current slope file
function! RunSlopeFile()
  let topline = line("w0")
  let botline = line("w$")
  call SaveAndRunSelection(topline,botline)
endfunction

" Function to run the current range/selection
function! RunVisualSelection() range
  let [line_start, column_start] = getpos("'<")[1:2]
  let [line_end, column_end] = getpos("'>")[1:2]
  call SaveAndRunSelection(line_start, line_end)
endfunction

" map sk and sj to find the next or prev '(' across lines
" in normal and visual mode
nnoremap <buffer> <localleader>j :call search('\V(')<cr>
nnoremap <buffer> <localleader>k :call search('\V(', 'b')<cr>
vnoremap <buffer> <localleader>j :call search('\V(')<cr>
vnoremap <buffer> <localleader>k :call search('\V(', 'b')<cr>

" map sr to run the file
nnoremap <buffer> <localleader>r :call RunSlopeFile()<cr>

" map sr to run the selection
vnoremap <buffer> <localleader>r :call RunVisualSelection()<cr>

" map sp to switch to the opposite paren
nnoremap <buffer> <localleader>p %
vnoremap <buffer> <localleader>p %

let b:did_ftplugin = 1
